`define ALUSHL          3'b101
`define ALUSHR          3'b110

/* SDLX machine defines */

/* Parameter declaration */
`define BYTE            7:0
`define DATAWIDTH       32
`define ADDRWIDTH       32
`define DATA            `DATAWIDTH-1:0
`define ADDR            `ADDRWIDTH-1:0
`define REGWIDTH        5
`define RDDR            `REGWIDTH-1:0
`define MAXREGS         1<<`REGWIDTH
`define MEMSIZE         4096

`define CLOCKPERIOD     20

/* Instruction fields */
`define OPCODEIDEX      IRIDEX[31:26]
`define CTLOPCODE	IR[31:26]
`define OPCODEFIELD	31:26
`define SRCREGFIELD	25:16
`define SRCRRS1FIELD	25:21
`define SRCRRS2FIELD	20:16
`define DSTREGFIELD	20:11
`define DSTIRDFIELD	20:16
`define DSTRRDFIELD	15:11
`define FUNCREGFIELD      5:0  /* New instruction field defination */
                               /* for "func" of R-Type ALU instructions. */ 

/* I-type Instruction */
`define IRD             IRMEMWB[20:16]

/* R-Type Instruction */
`define RRS1            IRIFID[25:21]
`define RRS2            IRIFID[20:16]
`define RRD             IRMEMWB[15:11]
`define FUNCIDEX        IRIDEX[5:0]
`define CTLFUNC		IR[5:0]

/* J-Type Instruction */
`define OFFSET          IRIFID[25:0]

/* control signals */
`define REGRW		WBMEMWB[0]
`define REGDST		WBMEMWB[1]
`define WBSEL		WBMEMWB[2]

`define MEMRW		MEMEXMEM[0]
`define JMPE		MEMIDEX[1]
`define BRAE		MEMIDEX[2]

`define ALUOPCODE	EXIDEX[`ALUO]
`define ZSEL		EXIDEX[2]
`define ALUINA		EXIDEX[3]
`define ALUINB		EXIDEX[4]

`define CTLREGRW	WBcontrol[0]
`define CTLREGDST       WBcontrol[1]
`define CTLWBSEL        WBcontrol[2]

`define CTLMEMRW        MEMcontrol[0]
`define CTLJMPE         MEMcontrol[1]
`define CTLBRAE         MEMcontrol[2]

`define CTLALUOPCODE    EXcontrol[`ALUO]
`define CTLZSEL         EXcontrol[2]
`define CTLALUINA       EXcontrol[3]
`define CTLALUINB       EXcontrol[4]

/* Address modes */
`define DSPLTYPE        0
`define IMMTYPE         1

/* OPcode map */

/* OPcodes */
`define NOP     `DATAWIDTH'b0

`define FSEL    6'b000000
`define J       6'b000010
`define BEQZ    6'b000100
`define BNEZ    6'b000101
`define ADDI    6'b001000
`define LW      6'b100011
`define SW      6'b101011
`define HLT     6'b001101

/* func */
`define AND     6'b100100
`define OR      6'b100101
`define SLL     6'b000000
`define SRL     6'b000010
`define ADD     6'b100000
`define SUB     6'b100010
`define MUL     6'b011000

/* Execution steps */
`define IDLE    0
`define IF      1
`define ID      2
`define EX      3
`define MEM     4
`define WB      5

/* Miscellaneous */
`define TRUE            1
`define FALSE           0

`define LEFT            1
`define RIGHT           0

`define DB_ON           debug = 1
`define DB_OFF          debug = 0

/* debug flag */
`define trc     1       // Trace mode:   print instruction information
`define vbs     0       // Verbose mode: print some more information
`define dbg     0       // Debug mode:   print excessive information



