// sdlx.v

//
// $Id: sdlx.v,v 1.1 1997/11/24 22:09:56 ritu Exp ritu $
//

//
// $Log: sdlx.v,v $
// Revision 1.1  1997/11/24 22:09:56  ritu
// Initial revision
//

`include "sdlx.h"
`include "alu.h"
`include "dbgflags.h"

module sdlx;    // top module

   
   reg     [31:0] IMEM[0:`MEMSIZE-1];
   reg 		  Reset;
   integer 	  i;
   
   initial
      begin
	 for(i=0;i<`MEMSIZE;i=i+1)
	    begin
	       IMEM[i]=32'b0;
	    end // initial begin
	       Reset = 0;
      end   
   wire            clk;
   
   wire [`ADDR]    pcin, npcin;
   wire [`DATA]    aluain, alubin, aluout;
   wire [`ALUO]    ALUOp;
   wire [`ALUC]    ALUCtrl;
   wire [`DATA]    wb2rf, rf2a, rf2b;
   wire [`RDDR]    rd;
   wire [`DATA]    dmemout;
   wire [`ADDR]    PC,NPC;
   wire [`DATA1]   IR;
   wire [`DATA]    I,iin;
   wire [`DATA]    A, B, ALUOut,
		   MD;
   wire 	   BC, bcin;

   Clock           Clock           (clk);

   Register32      IRReg           (IR, IMEM[PC],IRRW,clk);
   Register        PCReg           (PC, pcin, PCRW, clk);
   Register        NPCReg          (NPC, npcin, NPCRW, clk);
   Register        AReg            (A, rf2a, ARW, clk);
   Register        BReg            (B, rf2b, BRW, clk);
   Register        IReg            (I, iin, IRW, clk);
   Register        ALUOutReg       (ALUOut, aluout, ALUoutRW, clk);
   Register        MDReg           (MD, dmemout, MDRW, clk);
   OneBitReg       BCReg           (BC, bcin, BCRW, clk);

   RegFile         RegFile         (`RRS1, `RRS2, rd, wb2rf, rf2a, rf2b, RegRW, clk);
   DataMem         DataMem         (ALUOut, B, dmemout, MemRW, clk);
   Control         Control (
			    IR,
			    // Register control
			    IRRW,
			    PCRW,
			    NPCRW,
			    ARW,
			    BRW,
			    IRW,
			    ALUoutRW,
			    MDRW,
			    BCRW,
			    // Functional Units and MUXs
			    ZSel,
			    BraE,
			    JmpE,
			    RegDst,
			    ALUInA,
			    ALUInB,
			    SESel,
			    WBSel,
			    ALUOp,
			    RegRW,
			    MemRW,
			    
			    Reset,
			    clk) ;

   ALUControl      ALUControl      (ALUOp, `OPCODE, `FUNC, ALUCtrl);
   ALU             ALU             (aluain, alubin, aluout, ALUCtrl);

   SignExt         SignExt         (SESel, `OFFSET, iin);
   Add4            Add4            (PC, npcin);
   ZTest           ZTest           (ZSel, A, bcin);

   Mux             PCMux           ((BC & BraE) | JmpE, NPC, ALUOut, pcin);
   RddrMux         RDMux           (RegDst, `IRD, `RRD, rd);
   Mux             ALUAInMux       (ALUInA, NPC, A, aluain);
   Mux             ALUBInMux       (ALUInB, B, I, alubin);
   Mux             WBMux           (WBSel, MD, ALUOut, wb2rf);


   // Reset machine


   task apply_reset;
      begin
	 Reset = 1;
	 #`CLOCKPERIOD ;
	 Reset = 0;
	 PCReg.RegBuf = 0;
	 BCReg.RegBuf = 0;
      end
   endtask
   
   initial begin : load_prog
      integer i;
         
   //      addition with immediate operands - COMMENT ME OUT :)
   //      $readmemb("a.s", IMEM); 

   
   
   // addition with operands from data memory
   $readmemb("add.s", IMEM);
//   $readmemb("add.d", sdlx.DataMem.DMEM)pwd;
   
    if (`dbg) begin
       $display(" Loaded program instructions:");
       for(i=0; i<10; i=i+1)
	  $display(" %d> %b", i, IMEM[i]);
    end

    if (`dbg) begin
       $display(" Loaded program data:");
       for(i=0; i<10; i=i+1)
	  $display(" %d> %b", i, sdlx.DataMem.DMEM[i]);
    end
      
      if (`dbg)
	 $monitor($time, ":%b> PC=%h IR=%h   R1=%h R2=%h R3=%h   DMEM[0]=%h DMEM[1]=%h DMEM[2]=%h DMEM[3]=%h",
		  clk, PC, IR, RegFile.R1.RegBuf, 
		  RegFile.R2.RegBuf, RegFile.R3.RegBuf,
		  sdlx.DataMem.DMEM[0],
		  sdlx.DataMem.DMEM[1],
		  sdlx.DataMem.DMEM[2],
		  sdlx.DataMem.DMEM[3]);
      
      else if (`vbs)
	 $monitor($time, ":%b> %h %h %h %h %h %h %h %h %h",
		  clk, PC, NPC, IR, aluain, alubin, ALUOut, RegFile.R1.RegBuf,
		  RegFile.R2.RegBuf, RegFile.R3.RegBuf);
      
   //        $monitor($time, "> %h %h %h", PC, NPC, IR);
      
      apply_reset;
   end
   
   initial
      #600 $stop;
   
   
endmodule /* sdlx */

 

















